# Modules import
import c4d
from c4d import Document

def main():
	xpressoTag = c4d.BaseTag(1001149)
	c = GetCamera(doc)

	if (c != 'null'):
		# Get Active Camera values
		cPos = c[c4d.ID_BASEOBJECT_REL_POSITION]
		cRot = c[c4d.ID_BASEOBJECT_REL_ROTATION]
		cFocalDist = c[c4d.CAMERAOBJECT_TARGETDISTANCE]
		
		# Create Back Depth Offset USER DATA control
		backDepthUD = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
		backDepthUD[c4d.DESC_NAME] = "Back Depth Offset"
		backDepthUD[c4d.DESC_SHORT_NAME] = "Back  Depth Offset"
		#backDepthUD[c4d.DESC_DEFAULT] = 0  # DOES NOT WORK PROPERLY
		addBackDepthUD = c.AddUserData(backDepthUD)

		# Create Front Depth USER DATA control
		frontDepthUD = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
		frontDepthUD[c4d.DESC_NAME] = "Front Depth"
		frontDepthUD[c4d.DESC_SHORT_NAME] = "Front Depth"
		#frontDepthUD[c4d.DESC_DEFAULT] = 5 # DOES NOT WORK PROPERLY
		addFrontDepthUD = c.AddUserData(frontDepthUD)
		
		# Create Light Intensity USER DATA control
		lightIntensityUD = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
		lightIntensityUD[c4d.DESC_NAME] = "Light Intensity"
		lightIntensityUD[c4d.DESC_SHORT_NAME] = "Light Intensity"
		lightIntensityUD[c4d.DESC_UNIT] = c4d.DESC_UNIT_PERCENT
		lightIntensityUD[c4d.DESC_STEP] = 0.01
		lightIntensityUD[c4d.DESC_MIN] = 0
		lightIntensityUD[c4d.DESC_MAX] = 20
		#lightIntensityUD[c4d.DESC_DEFAULT] = 5  #WDOES NOT WORK PROPERLY
		addLightIntensityUD = c.AddUserData(lightIntensityUD)

		# Create New Area Light
		l = c4d.BaseObject(5102)
		l[c4d.ID_BASELIST_NAME] = 'Depth Lighting'
		l[c4d.LIGHT_TYPE] = 8
		l[c4d.LIGHT_BRIGHTNESS] = 5.0
		l[c4d.LIGHT_DETAILS_FALLOFF] = 8
		l[c4d.LIGHT_DETAILS_OUTERDISTANCE] = cFocalDist
		l[c4d.LIGHT_AREADETAILS_SIZEX] = 4
		l[c4d.LIGHT_AREADETAILS_SIZEY] = 4

		# Parent light to camera
		doc.InsertObject(l, parent=c)

		# Add Xpresso tag to light
		l.InsertTag(xpressoTag)

		# Get Xpresso tag master object
		xpressoTagGV = xpressoTag.GetNodeMaster()

		# Create main cam xpresso node and output ports for [Target Distance, Back Depth Offset, Front Depth, Light Intensity]
		createNewCamNode = xpressoTagGV.CreateNode(xpressoTagGV.GetRoot(), c4d.ID_OPERATOR_OBJECT, None, 0, 0 )
		createNewCamNode[c4d.GV_OBJECT_OBJECT_ID] = c

		addCamPortDistance = createNewCamNode.AddPort(c4d.GV_PORT_OUTPUT,c4d.DescID(c4d.DescLevel(c4d.CAMERAOBJECT_TARGETDISTANCE)))
		addCamPortUDBackDepth = createNewCamNode.AddPort(c4d.GV_PORT_OUTPUT,c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0), c4d.DescLevel(1)))
		addCamPortUDFrontDepth = createNewCamNode.AddPort(c4d.GV_PORT_OUTPUT,c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0), c4d.DescLevel(2)))
		addCamPortUDLightIntensity = createNewCamNode.AddPort(c4d.GV_PORT_OUTPUT,c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0), c4d.DescLevel(3)))
		
		# Create Light node and input ports for [Falloff Outer Distance, Falloff Inner Distance, Light Brightness]
		createNewLNode = xpressoTagGV.CreateNode(xpressoTagGV.GetRoot(), c4d.ID_OPERATOR_OBJECT, None, 500, 0 )
		createNewLNode[c4d.GV_OBJECT_OBJECT_ID] = l

		addLightPortOuterDist = createNewLNode.AddPort(c4d.GV_PORT_INPUT,c4d.DescID(c4d.DescLevel(c4d.LIGHT_DETAILS_OUTERDISTANCE)))
		addLightPortInnerDist = createNewLNode.AddPort(c4d.GV_PORT_INPUT,c4d.DescID(c4d.DescLevel(c4d.LIGHT_DETAILS_INNERDISTANCE)))
		addLightPortIntensity = createNewLNode.AddPort(c4d.GV_PORT_INPUT,c4d.DescID(c4d.DescLevel(c4d.LIGHT_BRIGHTNESS)))
		
		# Create Math node and set port variables
		createMathNode = xpressoTagGV.CreateNode(xpressoTagGV.GetRoot(), c4d.ID_OPERATOR_MATH, None, 200, -50)
		mathInPort1 = createMathNode.GetInPort(0)
		mathInPort2 = createMathNode.GetInPort(1)
		mathOutPort1 = createMathNode.GetOutPort(0)


		# Make port connections
		# Cam target distance > Math input 1
		addCamPortDistance.Connect(mathInPort1)
		# Back depth offset > Math input 2
		addCamPortUDBackDepth.Connect(mathInPort2)
		# Math output > Light outer distance
		mathOutPort1.Connect(addLightPortOuterDist)
		
		# UD Front depth control > light inner distance
		addLightPortInnerDist.Connect(addCamPortUDFrontDepth)
		
		# UD Light intensity control > light brightness
		addLightPortIntensity.Connect(addCamPortUDLightIntensity)
		
		#Update UI
		c4d.EventAdd()	
	

# FUNCTIONS
def GetCamera(doc):
	bd = doc.GetRenderBaseDraw()
	cp = bd.GetSceneCamera(doc)
	if cp:
		return cp
	else:
		return 'null'

# Launch script main function
if __name__=='__main__':
	main()